package org.processmining.dataawarecnetminer.extension.bindings;

import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.models.causalgraph.SimpleCausalGraph;

/**
 * Computation of the input- and output bindings of a C-Net based on a
 * collection of traces.
 * 
 * @author F. Mannhardt
 *
 */
public interface BindingsHeuristic {

	/**
	 * Computes the input- and output bindings for the C-Net.
	 * 
	 * @param causalGraph
	 * @throws MiningException
	 */
	void computeBindings(MinerContext context, SimpleCausalGraph causalGraph) throws MiningException;

	/**
	 * Returns the bindings of a C-Net for a given threshold.
	 * 
	 * @param config
	 * 
	 * @return the set of bindings
	 */
	Bindings getFilteredBindings(BindingsHeuristicConfig config);

}