package org.processmining.dataawarecnetminer.extension.conditionaldependencies;

import java.util.HashMap;
import java.util.Map;

import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.extension.HeuristicPlugin;
import org.processmining.dataawarecnetminer.interactive.methods.MethodConfiguration;
import org.processmining.dataawarecnetminer.model.DataAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.DataAwareCausalGraphImpl;
import org.processmining.dataawarecnetminer.model.DataRelation;
import org.processmining.dataawarecnetminer.model.DataRelationStorage;
import org.processmining.dataawarecnetminer.model.DependencyAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.DependencyRelation;
import org.processmining.dataawarecnetminer.model.DependencyRelationBinaryImpl;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.framework.plugin.annotations.KeepInProMCache;
import org.processmining.models.causalgraph.Relation;
import org.processmining.models.causalgraph.SimpleCausalGraph;

import com.google.common.collect.ImmutableMap;

@KeepInProMCache
@HeuristicPlugin
public class ConditionalDependencyHeuristicExtensionNoneImpl extends DefaultConditionalDependencyHeuristicFactory {

	public ConditionalDependencyHeuristic newInstance(EventRelationStorage eventStorage,
			DataRelationStorage dataStorage) {
		return new ConditionalDependencyHeuristic() {

			public DataAwareCausalGraph computeDependencies(MinerContext context, MethodConfiguration configuration,
					SimpleCausalGraph causalGraph) throws MiningException {
				return new DataAwareCausalGraphImpl(causalGraph.getSetActivities(),
						convertOriginalRelations(causalGraph), ImmutableMap.<Relation, DataRelation>of());
			}

			private Map<Relation, DependencyRelation> convertOriginalRelations(SimpleCausalGraph causalGraph) {
				Map<Relation, DependencyRelation> dependencyRelations = new HashMap<>();
				if (causalGraph instanceof DependencyAwareCausalGraph) {
					DependencyAwareCausalGraph dependencyAwareCausalGraph = (DependencyAwareCausalGraph) causalGraph;
					for (Relation relation : causalGraph.getCausalRelations()) {
						dependencyRelations.put(relation, dependencyAwareCausalGraph.getDependencyRelation(relation));
					}
				} else {
					DependencyRelation binaryRelation = new DependencyRelationBinaryImpl();
					for (Relation relation : causalGraph.getCausalRelations()) {
						dependencyRelations.put(relation, binaryRelation);
					}
				}
				return dependencyRelations;
			}

		};
	}

	public String getName() {
		return "None";
	}

	public String getReference() {
		return "No conditional dependencies are computed.";
	}

}
