package org.processmining.dataawarecnetminer.extension.conditionaldependencies;

import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.extension.conditionaldependencies.ConditionalDependencyHeuristicConfig.QualityMeasure;
import org.processmining.dataawarecnetminer.interactive.methods.MethodConfiguration;
import org.processmining.dataawarecnetminer.mining.data.DataAwareCausalGraphBuilder;
import org.processmining.dataawarecnetminer.model.DataAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.DataRelationStorage;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.datadiscovery.RuleDiscoveryException;
import org.processmining.models.causalgraph.SimpleCausalGraph;

public class QualityAwareConditionalDependencyHeuristicImpl extends DefaultConditionalDependencyHeuristic {

	protected DataAwareCausalGraphBuilder causalGraphBuilder;
	private QualityMeasure qualityMeasure;

	public QualityAwareConditionalDependencyHeuristicImpl(EventRelationStorage eventStorage, DataRelationStorage dataStorage,
			QualityMeasure qualityMeasure) {
		super();
		this.qualityMeasure = qualityMeasure;
		this.causalGraphBuilder = new DataAwareCausalGraphBuilder(eventStorage, dataStorage);
	}

	public DataAwareCausalGraph computeDependencies(MinerContext context, MethodConfiguration configuration,
			SimpleCausalGraph causalGraph) throws MiningException {
		causalGraphBuilder.setConfig(configureDataCausalGraphBuilder(configuration));
		try {
			return causalGraphBuilder.build(context, causalGraph);
		} catch (RuleDiscoveryException e) {
			throw new MiningException(e);
		}
	}

	protected ConditionalDependencyHeuristicConfig configureDataCausalGraphBuilder(MethodConfiguration methodConfig) {
		ConditionalDependencyHeuristicConfig config = new ConditionalDependencyHeuristicConfig();
		config.setDependencyThreshold(methodConfig.getDependencyThreshold());
		config.setRelativeToBestThreshold(methodConfig.getRelativeToBestThreshold());
		config.setObservationThreshold(methodConfig.getFrequencyThreshold());
		config.setAllTasksConnected(methodConfig.isAllTasksConnected());
		config.setAcceptedTasksConnected(methodConfig.isAcceptedTasksConnected());
		config.setGuardThreshold(methodConfig.getConditionThreshold());
		config.setQualityMeasure(qualityMeasure);

		config.setDataDiscoveryConfig(methodConfig.getDataDiscoveryConfig());
		config.setDataDiscoveryAttributes(methodConfig.getDataDiscoveryAttributes());
		return config;
	}

}