package org.processmining.dataawarecnetminer.extension.dependencies.causalmatrix;

import java.util.HashMap;
import java.util.Map;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.impl.XAttributeMapImpl;
import org.deckfour.xes.model.impl.XLogImpl;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.causalactivitygraphcreator.creators.GraphCreator;
import org.processmining.causalactivitygraphcreator.creators.GraphCreatorParameters;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.causalactivitymatrixminer.miners.MatrixMiner;
import org.processmining.causalactivitymatrixminer.miners.MatrixMinerParameters;
import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.extension.dependencies.DefaultDependencyHeuristic;
import org.processmining.dataawarecnetminer.interactive.methods.MethodConfiguration;
import org.processmining.dataawarecnetminer.model.DependencyAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.DependencyAwareCausalGraphImpl;
import org.processmining.dataawarecnetminer.model.DependencyRelation;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.models.causalgraph.Relation;

public abstract class DefaultCausalMatrixDependencyHeuristic extends DefaultDependencyHeuristic {

	protected final EventRelationStorage eventStorage;

	public DefaultCausalMatrixDependencyHeuristic(EventRelationStorage eventStorage) {
		super();
		this.eventStorage = eventStorage;
	}

	public DependencyAwareCausalGraph computeDependencies(MinerContext context, MethodConfiguration config) {
		XLog log = wrapInLog();
		MatrixMinerParameters matrixParameters = new MatrixMinerParameters(log); //TODO why is the standard constructor deprecated?
		matrixParameters.setClassifier(eventStorage.getClassifier());
		CausalActivityMatrix matrix = createMatrixMiner().mineMatrix(context.getPluginContext(), log, matrixParameters);
		GraphCreator creator = createGraphCreator();
		final CausalActivityGraph activityGraph = creator.apply(context.getPluginContext(), matrix,
				new GraphCreatorParameters());

		Map<Relation, DependencyRelation> dependencyRelations = new HashMap<>();
		for (Relation relation : activityGraph.getCausalRelations()) {
			final double causality = activityGraph.getCausality(relation.getSource(), relation.getTarget());
			if (causality >= config.getDependencyThreshold()) {
				double requiredObservations = config.getFrequencyThreshold() * eventStorage.countTraces();
				if (eventStorage.countDirectlyFollows(relation) >= requiredObservations) {
					dependencyRelations.put(relation, new DependencyRelation() {

						public double getDependencyValue() {
							return causality;
						}
					});
				}
			}
		}
		return new DependencyAwareCausalGraphImpl(activityGraph.getSetActivities(), dependencyRelations);
	}

	abstract protected GraphCreator createGraphCreator();

	abstract protected MatrixMiner createMatrixMiner();

	private XLog wrapInLog() {
		XLogImpl log = new XLogImpl(new XAttributeMapImpl());
		log.addAll(eventStorage.getAllTraces());
		return log;
	}

}