package org.processmining.dataawarecnetminer.interactive;

import java.awt.BorderLayout;
import java.awt.Component;
import java.text.DateFormat;
import java.text.MessageFormat;
import java.util.Date;

import javax.swing.Box;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.processmining.dataawarecnetminer.interactive.methods.MiningMethod;
import org.processmining.dataawarecnetminer.model.DataRelationStorage;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.framework.plugin.PluginContext;

public class InteractiveDataAwareCausalMinerController extends JPanel implements HeuristicMinerController {

	private static final long serialVersionUID = 1L;

	private final ClosableTabbedPane tabs;

	private PluginContext context;

	private InteractiveDataAwareCausalMinerVisualization lastVisualization;
	private boolean isUpdating = false;

	public InteractiveDataAwareCausalMinerController(final PluginContext context,
			final EventRelationStorage eventStorage, final DataRelationStorage dataStorage) {
		super(new BorderLayout());
		this.context = context;
		tabs = new ClosableTabbedPane();

		InteractiveDataAwareCausalMinerVisualization minerVisualization = new InteractiveDataAwareCausalMinerVisualization(
				this, eventStorage, dataStorage);
		lastVisualization = minerVisualization;
		tabs.addNonClosableTabe("Base Model", minerVisualization, "");

		final Component tabCreateNew = Box.createGlue();
		tabs.addNonClosableTabe("+", tabCreateNew, "Create a copy of the current settings");
		tabs.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				if (isUpdating == true) {
					return;
				}

				if (tabs.getSelectedComponent() == tabCreateNew && lastVisualization != null) {
					isUpdating = true;
					try {
						int indexOfTab = tabs.indexOfComponent(tabCreateNew);
						tabs.removeTabAt(indexOfTab);
						tabs.setSelectedComponent(lastVisualization);

						MiningMethod method = lastVisualization.getCurrentMiningMethod();
						lastVisualization.updateConfiguration(method);

						String defaultName = MessageFormat.format("{0}",
								DateFormat.getDateTimeInstance().format(new Date()));
						InteractiveDataAwareCausalMinerVisualization newTab = new InteractiveDataAwareCausalMinerVisualization(
								InteractiveDataAwareCausalMinerController.this, eventStorage, dataStorage,
								method.getClass(), method.getConfiguration());
						//TODO retain query
						String newName = JOptionPane.showInputDialog(tabs, "Please specify a name for the new tab",
								defaultName);
						tabs.addClosableTab(newName, newTab, "");
						tabs.addNonClosableTabe("+", tabCreateNew, "Create copy of current result");
						tabs.setSelectedComponent(newTab);
					} finally {
						isUpdating = false;
					}
				} else if (tabs.getSelectedComponent() instanceof InteractiveDataAwareCausalMinerVisualization) {
					lastVisualization = (InteractiveDataAwareCausalMinerVisualization) tabs.getSelectedComponent();
				}

			}
		});

		add(tabs, BorderLayout.CENTER);
	}

	public PluginContext getContext() {
		return context;
	}

}
