package org.processmining.dataawarecnetminer.interactive;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog.ModalityType;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.DateFormat;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Set;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.SwingWorker;
import javax.swing.border.CompoundBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.util.XAttributeUtils;
import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.extension.HeuristicExtension;
import org.processmining.dataawarecnetminer.extension.HeuristicRegistry;
import org.processmining.dataawarecnetminer.extension.bindings.BindingsHeuristicExtension;
import org.processmining.dataawarecnetminer.extension.conditionaldependencies.ConditionalDependencyHeuristicExtension;
import org.processmining.dataawarecnetminer.extension.conformance.ConformanceExtensionAlignmentImpl;
import org.processmining.dataawarecnetminer.extension.conformance.ConformanceHeuristicExtension;
import org.processmining.dataawarecnetminer.extension.decision.DecisionHeuristicExtension;
import org.processmining.dataawarecnetminer.extension.dependencies.DependencyHeuristicExtension;
import org.processmining.dataawarecnetminer.interactive.methods.MethodConfiguration;
import org.processmining.dataawarecnetminer.interactive.methods.MiningMethod;
import org.processmining.dataawarecnetminer.interactive.methods.cnet.DataAwareCNetMethod;
import org.processmining.dataawarecnetminer.interactive.methods.cnet.DataAwareDCNetMethod;
import org.processmining.dataawarecnetminer.interactive.methods.cnet.DataAwareDPNMinerMethod;
import org.processmining.dataawarecnetminer.interactive.methods.cnet.DataAwarePetrinetMinerMethod;
import org.processmining.dataawarecnetminer.interactive.methods.cnet.DataDependencyGraphMethod;
import org.processmining.dataawarecnetminer.interactive.methods.relations.DependencyRelationMethod;
import org.processmining.dataawarecnetminer.interactive.methods.relations.DirectlyFollowsMethod;
import org.processmining.dataawarecnetminer.interactive.methods.relations.L1DependencyMethod;
import org.processmining.dataawarecnetminer.interactive.methods.relations.L2DependencyMethod;
import org.processmining.dataawarecnetminer.interactive.methods.relations.LongDistanceDependencyMethod;
import org.processmining.dataawarecnetminer.model.DataRelationStorage;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.dataawarecnetminer.util.ExtendedDoubleSlider;
import org.processmining.dataawarecnetminer.visualization.ColorScheme;
import org.processmining.dataawarecnetminer.visualization.DataDiscoveryConfigurationPanel;
import org.processmining.datadiscovery.estimators.Type;
import org.processmining.datadiscovery.estimators.weka.WekaUtil;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.util.ui.widgets.ColorSchemeLegend;
import org.processmining.framework.util.ui.widgets.ProMAutoCompletingTextField;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.framework.util.ui.widgets.ProMScrollPane;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.models.causalgraph.Relation;
import org.processmining.plugins.DataConformance.visualization.DataAwareStepTypes;
import org.processmining.xeslite.query.AttributeTypeResolver;
import org.processmining.xeslite.query.XIndex;
import org.processmining.xeslite.query.syntax.ParseException;

import com.fluxicon.slickerbox.components.NiceDoubleSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.google.common.base.Stopwatch;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import com.google.common.collect.Multiset.Entry;

public class InteractiveDataAwareCausalMinerVisualization extends JPanel {

	private static class AttributeMenuItem extends JMenuItem {

		private static final long serialVersionUID = -6335245188318559300L;

		protected final String key;
		protected final String value;

		public AttributeMenuItem(String key, String value) {
			super(value);
			this.key = key;
			this.value = value;
		}

		public String getQuery(String operator) {
			return key + operator + "\"" + value + "\"";
		}

	}

	private static class BooleanAttributeMenuItem extends AttributeMenuItem {

		private static final long serialVersionUID = -7278055543787747503L;

		public BooleanAttributeMenuItem(String key, String value) {
			super(key, value);
		}

		public String getQuery(String operator) {
			return key + operator + value;
		}

	}

	private static class OtherAttributeMenuItem extends AttributeMenuItem {

		private static final long serialVersionUID = 464854826509195679L;

		public OtherAttributeMenuItem(String text, String key) {
			super(key, null);
			setText(text);
		}

		public String getQuery(String operator) {
			return key;
		}

	}

	private static final String QUERY_HELP_TEXT = "<HTML><h1>Query Syntax</h1>"
			+ "SQL-like filtering by trace/event names (event occurrence) or by attributes (attribute with specified value present in trace)"
			+ "<h2>Examples</h2>" + "<ul>" + "<li>'A' - searches for traces that contain event with exact name 'A'</li>"
			+ "<li>'\"event A\"' - searches for traces that contain event with exact name 'event A'</li>"
			+ "<li>'%A' - searches for traces that contain event whose name contains 'A'</li>"
			+ "<li>'~.*A.*' - searches for traces that contain event whose name matches the regex '.*A.*'</li>"
			+ "<li>'amount>50' - searches for traces that contain events with numeric attribute 'amount' which is greater than 50</li>"
			+ "<li>'name%joe' - searches for traces that contain events with literal attribute 'name' which contains the value 'joe'</li>"
			+ "</ul>Omit the single quotes in all examples." + "<h2>Details</h2>" + "<ul>"
			+ "<li>Either searches for 'concept:name' attributes of traces and events (start with '~' for use a regular expression, start with '%' to use a 'contains' query)</li>"
			+ "<li>Supports filtering by trace/event attributes in form of 'eventName'.'attributeName OP attributeValue'.</li>"
			+ "<li>Supported operators (OP) are (=, >, <, !=, >=, <=, % (contains), ~ (regex), some operators only work with numeric/date attributes.</li>"
			+ "<li>Terms can be connected with (AND, OR) and nested with parens.</li>" + "</ul></HTML>";

	private static final int DEFAULT_HEIGHT = 800;
	private static final int DEFAULT_WIDTH = 1000;

	public static void showCustom(Component parentComponent, JComponent component, String dialogTitle,
			ModalityType modalityType) {
		ProMScrollPane scrollPane = new ProMScrollPane(component);
		scrollPane.setBorder(BorderFactory.createLineBorder(Color.BLACK, 2));
		if (component.getPreferredSize().getHeight() > DEFAULT_HEIGHT
				|| component.getPreferredSize().getWidth() > DEFAULT_WIDTH) {
			scrollPane.setPreferredSize(new Dimension(DEFAULT_WIDTH, DEFAULT_HEIGHT));
		}
		scrollPane.setMaximumSize(new Dimension(DEFAULT_WIDTH, DEFAULT_HEIGHT));
		scrollPane.setAlignmentX(JComponent.TOP_ALIGNMENT);
		Object[] options = { "OK" };
		JOptionPane optionPane = new JOptionPane(scrollPane, JOptionPane.PLAIN_MESSAGE, JOptionPane.DEFAULT_OPTION,
				null, options, options[0]);
		JDialog dialog = optionPane.createDialog(parentComponent, dialogTitle);
		dialog.setResizable(true);
		dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		dialog.setModalityType(modalityType);
		dialog.pack();
		dialog.setVisible(true);
	}

	private static final long serialVersionUID = 2460846671787453638L;

	private static final class AttributeTypeResolverNaiveImpl implements AttributeTypeResolver {

		private final Iterable<XTrace> traces;

		private AttributeTypeResolverNaiveImpl(Iterable<XTrace> traces) {
			this.traces = traces;
		}

		public Class<?> getAttributeType(String attributeName) {
			// simply return the first class found
			for (XTrace t : traces) {
				for (XEvent e : t) {
					XAttribute attribute = e.getAttributes().get(attributeName);
					if (attribute != null) {
						return XAttributeUtils.getType(attribute);
					}
				}
			}
			return XAttributeLiteral.class;
		}
	}

	private final MiningMethod[] standardMiningOutputs = { new DirectlyFollowsMethod(), new DataDependencyGraphMethod(),
			new DataAwareCNetMethod(), new DataAwareDCNetMethod(), new DataAwareDPNMinerMethod(),
			new DataAwarePetrinetMinerMethod() };

	private final MiningMethod[] expertMiningOutputs = { new DirectlyFollowsMethod(), new DependencyRelationMethod(),
			new L1DependencyMethod(), new L2DependencyMethod(), new LongDistanceDependencyMethod(),
			new DataDependencyGraphMethod(), new DataAwareCNetMethod(), new DataAwareDCNetMethod(),
			new DataAwareDPNMinerMethod(), new DataAwarePetrinetMinerMethod() };

	private final Collection<XTrace> originalTraces;
	private final long originalEventCount;

	private EventRelationStorage eventStorage;
	private DataRelationStorage dataStorage;

	private final ProMAutoCompletingTextField searchQuery;

	private final JPanel configPanel;
	private final ProMComboBox<MiningMethod> outputCombo;

	private final NiceDoubleSlider dependencySlider;
	private final NiceDoubleSlider l1Slider;
	private final NiceDoubleSlider l2Slider;
	private final NiceDoubleSlider relativeToBestSlider;
	private final NiceDoubleSlider frequencySlider;
	private final NiceDoubleSlider bindingSlider;
	private final NiceDoubleSlider conditionSlider;
	private final NiceDoubleSlider decisionSlider;

	private final JCheckBox longTermCheckbox;
	private final JCheckBox allConnectedCheckbox;
	private final JCheckBox acceptedConnectedCheckbox;
	private final JCheckBox noUnconnectedCheckbox;

	private final DataDiscoveryConfigurationPanel dataConfigurationPanel;
	private final JButton configureClassification;

	private final JButton exportButton;
	private final JCheckBox expertCheckbox;

	private final JPanel wrapper;
	private boolean isUpdating;

	private JLabel searchResultLabel;

	private final ProMComboBox<DependencyHeuristicExtension> dependencyHeuristic;
	private final ProMComboBox<ConditionalDependencyHeuristicExtension> dataDependencyHeuristic;
	private final ProMComboBox<BindingsHeuristicExtension> bindingHeuristic;
	private final ProMComboBox<ConformanceHeuristicExtension> conformanceHeuristic;

	private final JProgressBar progressBar;
	private final JButton cancelButton;

	private final InteractiveProgressListener progressListener;

	private JLabel bindingsHeuristicLabel;

	private JLabel dataDependencyHeuristicLabel;

	private JLabel dependencyHeuristicLabel;

	private JLabel conformanceHeuristicLabel;

	private JLabel thresholdResultLabel;

	private JLabel decisionHeuristicLabel;

	private HeuristicMinerController controller;

	private HeuristicRegistry heuristicRegistry;

	private JPanel alignmentLegend;

	private ProMComboBox<DecisionHeuristicExtension> decisionHeuristic;

	public InteractiveDataAwareCausalMinerVisualization(final HeuristicMinerController controller,
			final EventRelationStorage eventStorage, final DataRelationStorage dataStorage) {
		this(controller, eventStorage, dataStorage, DataAwareCNetMethod.class,
				new MethodConfiguration().setAllTasksConnected(false).setAcceptedTasksConnected(true));
	}

	@SuppressWarnings("unchecked")
	public InteractiveDataAwareCausalMinerVisualization(final HeuristicMinerController controller,
			final EventRelationStorage eventStorage, final DataRelationStorage dataStorage,
			Class<? extends MiningMethod> miningMethod, MethodConfiguration config) {
		this.controller = controller;
		this.dataStorage = dataStorage;
		this.eventStorage = eventStorage;
		this.originalTraces = eventStorage.getAllTraces();
		this.originalEventCount = eventStorage.countEvents();
		setLayout(new BorderLayout());

		dataConfigurationPanel = new DataDiscoveryConfigurationPanel(dataStorage.getAvailableAttributes().keySet(),
				this);

		wrapper = new JPanel(new BorderLayout());
		wrapper.setBackground(Color.WHITE);
		wrapper.setOpaque(true);
		add(wrapper, BorderLayout.CENTER);

		progressBar = new JProgressBar();
		progressBar.setIndeterminate(true);
		progressBar.setStringPainted(true);
		progressBar.setString("processing ... ");
		progressListener = new InteractiveProgressListener(progressBar);
		cancelButton = SlickerFactory.instance().createButton("cancel");
		cancelButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				progressListener.cancel();
			}
		});

		searchQuery = new ProMAutoCompletingTextField(config.getFilterQuery() != null ? config.getFilterQuery() : "",
				"attribute = \"value\" (use right click to view available attributes)");
		searchQuery.setDictionary(ImmutableList.copyOf(dataStorage.getAvailableAttributes().keySet()));
		searchQuery.setMinimumSize(new Dimension(200, 25));
		searchQuery.setPreferredSize(new Dimension(200, 25));
		searchQuery.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				update();
			}
		});
		createAttributeHelper(dataStorage, searchQuery.getTextField());

		JButton helpButton = SlickerFactory.instance().createButton("?");
		helpButton.setMinimumSize(new Dimension(30, 25));
		helpButton.setPreferredSize(new Dimension(30, 25));
		helpButton.setMaximumSize(new Dimension(30, 25));
		helpButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				JEditorPane pane = new JEditorPane("text/html", QUERY_HELP_TEXT);
				pane.setEditable(false);
				showCustom(wrapper, pane, "Query Syntax Help", ModalityType.MODELESS);
			}

		});
		helpButton.setToolTipText(QUERY_HELP_TEXT);

		searchResultLabel = new JLabel(MessageFormat.format("Using {0}/{0} traces, {1}/{1} events",
				originalTraces.size(), eventStorage.countEvents()));

		frequencySlider = new ExtendedDoubleSlider("Frequency", 0.0d, 1.0d, config.getFrequencyThreshold(),
				Orientation.VERTICAL);
		frequencySlider.setToolTipText("<html>Observation frequency threshold used in step A<br/>"
				+ "(0.1 means only directly-follows relations observed in more than 10% of all the traces are included)</html>");
		frequencySlider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					update();
				}
			}

		});
		dependencySlider = new ExtendedDoubleSlider("Dependency", 0.0d, 1.0d, config.getDependencyThreshold(),
				Orientation.VERTICAL);
		dependencySlider.setToolTipText("<html>Dependency measure threshold used in step A and B</html>");
		dependencySlider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					update();
				}
			}

		});
		l1Slider = new ExtendedDoubleSlider("L1 Loop", 0.0d, 1.0d, config.getL1Threshold(), Orientation.VERTICAL);
		l1Slider.setToolTipText(
				"<html>Dependency measure threshold for loops of lenghth one (abbbc) used in step A</html>");
		l1Slider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					update();
				}
			}

		});
		l2Slider = new ExtendedDoubleSlider("L2 Loop", 0.0d, 1.0d, config.getL2Threshold(), Orientation.VERTICAL);
		l2Slider.setToolTipText(
				"<html>Dependency measure threshold for loops of lenghth two (abcbcbcd) used in step A</html>");
		l2Slider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					update();
				}
			}

		});
		relativeToBestSlider = new ExtendedDoubleSlider("Relative To Best", 0.0d, 1.0d,
				config.getRelativeToBestThreshold(), Orientation.VERTICAL);
		relativeToBestSlider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					update();
				}
			}

		});
		bindingSlider = new ExtendedDoubleSlider("Bindings", 0.0d, 1.0d, config.getBindingsThreshold(),
				Orientation.VERTICAL);
		bindingSlider.setToolTipText("<html>Binding frequency threshold used in step C</html>");
		bindingSlider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					update();
				}
			}

		});

		conditionSlider = new ExtendedDoubleSlider("Conditions", 0.0d, 1.0d, config.getConditionThreshold(),
				Orientation.VERTICAL);
		conditionSlider.setToolTipText("<html>Condition quality threshold used in step B</html>");
		conditionSlider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					update();
				}
			}

		});

		decisionSlider = new ExtendedDoubleSlider("Decisions", 0.0d, 1.0d, config.getDecisionThreshold(),
				Orientation.VERTICAL);
		decisionSlider.setToolTipText("<html>Decision rules quality threshold used in step D</html>");
		decisionSlider.getSlider().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					update();
				}
			}

		});

		outputCombo = new ProMComboBox<MiningMethod>(standardMiningOutputs);
		outputCombo.setPreferredSize(null);
		outputCombo.setMinimumSize(new Dimension(150, 25));
		outputCombo.setMaximumSize(new Dimension(200, 25));
		selectMethod(miningMethod);
		outputCombo.setSelectedItem(miningMethod);
		outputCombo.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				update();
			}
		});

		longTermCheckbox = SlickerFactory.instance().createCheckBox("Long Term Dependencies", config.isLongTerm());
		longTermCheckbox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				update();
			}
		});
		allConnectedCheckbox = SlickerFactory.instance().createCheckBox("All Tasks Connected",
				config.isAllTasksConnected());
		allConnectedCheckbox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				update();
			}
		});
		acceptedConnectedCheckbox = SlickerFactory.instance().createCheckBox("Accepted Connected",
				config.isAcceptedTasksConnected());
		acceptedConnectedCheckbox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				update();
			}
		});
		noUnconnectedCheckbox = SlickerFactory.instance().createCheckBox("No Unconnected",
				config.isNoTasksUnconnected());
		noUnconnectedCheckbox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				update();
			}
		});

		expertCheckbox = SlickerFactory.instance().createCheckBox("Expert Options", false);
		expertCheckbox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				updateConfigUI(getCurrentMiningMethod());
				isUpdating = true;
				try {
					Class<? extends Object> methodClass = outputCombo.getSelectedItem().getClass();
					if (expertCheckbox.isSelected()) {
						outputCombo.setModel(new DefaultComboBoxModel<MiningMethod>(expertMiningOutputs));
					} else {
						outputCombo.setModel(new DefaultComboBoxModel<MiningMethod>(standardMiningOutputs));
					}
					selectMethod(methodClass);
				} finally {
					isUpdating = false;
				}

			}
		});

		configureClassification = SlickerFactory.instance().createButton("Data-aware Config");
		configureClassification.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				showCustom(InteractiveDataAwareCausalMinerVisualization.this, dataConfigurationPanel,
						"Configure Data-aware Discovery", ModalityType.MODELESS);
			}
		});

		exportButton = SlickerFactory.instance().createButton("Export model");
		exportButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				MiningMethod miningMethod = getCurrentMiningMethod();
				boolean needsDataUpdate = updateConfiguration(miningMethod);
				if (needsDataUpdate) {
					miningMethod.updateData(eventStorage, dataStorage);
				}
				try {
					String defaultName = MessageFormat.format("{0} @ {1}", getCurrentMiningMethod(),
							DateFormat.getDateTimeInstance().format(new Date()));
					final ExecutorService executor = Executors
							.newFixedThreadPool(Runtime.getRuntime().availableProcessors());
					try {
						miningMethod.save(controller.getContext(),
								JOptionPane.showInputDialog(InteractiveDataAwareCausalMinerVisualization.this,
										"Please specify the name for the exported model", defaultName));
					} finally {
						executor.shutdown();
					}

				} catch (MiningException e1) {
					ProMUIHelper.showErrorMessage(InteractiveDataAwareCausalMinerVisualization.this,
							"Could not save current model", "Error mining", e1);
				}
			}
		});

		heuristicRegistry = new HeuristicRegistry();

		DefaultComboBoxModel<DependencyHeuristicExtension> dependencyGraphModel = new DefaultComboBoxModel<DependencyHeuristicExtension>();
		for (DependencyHeuristicExtension dependencyHeuristic : heuristicRegistry
				.getHeuristicExtensions(controller.getContext(), DependencyHeuristicExtension.class)) {
			dependencyGraphModel.addElement(dependencyHeuristic);
		}
		dependencyHeuristic = new ProMComboBox<DependencyHeuristicExtension>(dependencyGraphModel);
		dependencyHeuristic.setRenderer(createHeuristicExtensionRenderer());
		dependencyHeuristic.setPreferredSize(new Dimension(200, 25));
		dependencyHeuristic.setMinimumSize(new Dimension(200, 25));
		dependencyHeuristic.setMaximumSize(new Dimension(200, 25));
		dependencyHeuristic.setSelectedItem(config.getDependencyHeuristicExtension());
		dependencyHeuristic.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				update();
			}
		});

		DefaultComboBoxModel<ConditionalDependencyHeuristicExtension> dataDependencyGraphModel = new DefaultComboBoxModel<ConditionalDependencyHeuristicExtension>();
		for (ConditionalDependencyHeuristicExtension dataHeuristic : heuristicRegistry
				.getHeuristicExtensions(controller.getContext(), ConditionalDependencyHeuristicExtension.class)) {
			dataDependencyGraphModel.addElement(dataHeuristic);
		}
		dataDependencyHeuristic = new ProMComboBox<ConditionalDependencyHeuristicExtension>(dataDependencyGraphModel);
		dataDependencyHeuristic.setRenderer(createHeuristicExtensionRenderer());
		dataDependencyHeuristic.setPreferredSize(new Dimension(200, 25));
		dataDependencyHeuristic.setMinimumSize(new Dimension(200, 25));
		dataDependencyHeuristic.setMaximumSize(new Dimension(200, 25));
		dataDependencyHeuristic.setSelectedItem(config.getDataDependencyHeuristicExtension());
		dataDependencyHeuristic.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				update();
			}
		});

		DefaultComboBoxModel<BindingsHeuristicExtension> bindingModel = new DefaultComboBoxModel<BindingsHeuristicExtension>();
		for (BindingsHeuristicExtension bindingsHeuristic : heuristicRegistry
				.getHeuristicExtensions(controller.getContext(), BindingsHeuristicExtension.class)) {
			bindingModel.addElement(bindingsHeuristic);
		}

		bindingHeuristic = new ProMComboBox<BindingsHeuristicExtension>(bindingModel);
		bindingHeuristic.setRenderer(createHeuristicExtensionRenderer());
		bindingHeuristic.setPreferredSize(new Dimension(200, 25));
		bindingHeuristic.setMinimumSize(new Dimension(200, 25));
		bindingHeuristic.setMaximumSize(new Dimension(200, 25));
		bindingHeuristic.setSelectedItem(config.getBindingsHeuristicExtension());
		bindingHeuristic.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				update();
			}
		});

		DefaultComboBoxModel<ConformanceHeuristicExtension> conformanceGraphModel = new DefaultComboBoxModel<ConformanceHeuristicExtension>();
		for (ConformanceHeuristicExtension conformanceHeuristic : heuristicRegistry
				.getHeuristicExtensions(controller.getContext(), ConformanceHeuristicExtension.class)) {
			conformanceGraphModel.addElement(conformanceHeuristic);
		}
		conformanceHeuristic = new ProMComboBox<ConformanceHeuristicExtension>(conformanceGraphModel);
		conformanceHeuristic.setRenderer(createHeuristicExtensionRenderer());
		conformanceHeuristic.setPreferredSize(new Dimension(200, 25));
		conformanceHeuristic.setMinimumSize(new Dimension(200, 25));
		conformanceHeuristic.setMaximumSize(new Dimension(200, 25));
		conformanceHeuristic.setSelectedItem(config.getConformanceHeuristicExtension());
		conformanceHeuristic.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				update();
			}
		});

		DefaultComboBoxModel<DecisionHeuristicExtension> decisionGraphModel = new DefaultComboBoxModel<DecisionHeuristicExtension>();
		for (DecisionHeuristicExtension conformanceHeuristic : heuristicRegistry
				.getHeuristicExtensions(controller.getContext(), DecisionHeuristicExtension.class)) {
			decisionGraphModel.addElement(conformanceHeuristic);
		}
		decisionHeuristic = new ProMComboBox<DecisionHeuristicExtension>(decisionGraphModel);
		decisionHeuristic.setRenderer(createHeuristicExtensionRenderer());
		decisionHeuristic.setPreferredSize(new Dimension(200, 25));
		decisionHeuristic.setMinimumSize(new Dimension(200, 25));
		decisionHeuristic.setMaximumSize(new Dimension(200, 25));
		decisionHeuristic.setSelectedItem(config.getDecisionHeuristicExtension());
		decisionHeuristic.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				update();
			}
		});

		JPanel sliderPanel = new JPanel();
		sliderPanel.setLayout(new BoxLayout(sliderPanel, BoxLayout.X_AXIS));
		sliderPanel.add(frequencySlider);
		sliderPanel.add(dependencySlider);
		sliderPanel.add(bindingSlider);
		sliderPanel.add(conditionSlider);
		sliderPanel.add(decisionSlider);
		sliderPanel.add(l1Slider);
		sliderPanel.add(l2Slider);
		sliderPanel.add(relativeToBestSlider);
		
		JPanel checkboxPanel = new JPanel();
		checkboxPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 5;
		c.gridy = 0;
		checkboxPanel.add(longTermCheckbox, c);
		checkboxPanel.add(configureClassification, c);
		c.gridy = 1;
		checkboxPanel.add(allConnectedCheckbox, c);
		checkboxPanel.add(expertCheckbox, c);
		c.gridy = 2;
		checkboxPanel.add(acceptedConnectedCheckbox, c);

		configPanel = new JPanel();
		configPanel.setLayout(new GridBagLayout());
		c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.anchor = GridBagConstraints.CENTER;
		c.weightx = 1;
		c.weighty = 0;
		c.gridx = 0;

		configPanel.add(new JLabel("<HTML><B>INPUT: TRACES</B></HTML>"), c);
		addSpacer(configPanel, c);
		JPanel searchPanel = new JPanel();
		searchPanel.setLayout(new BoxLayout(searchPanel, BoxLayout.X_AXIS));
		searchPanel.add(searchQuery);
		searchPanel.add(helpButton);
		configPanel.add(searchPanel, c);
		configPanel.add(searchResultLabel, c);
		configPanel.add(Box.createVerticalStrut(10), c);

		configPanel.add(new JLabel("<HTML><B>OUTPUT: PROCESS MODEL</B></HTML>"), c);
		addSpacer(configPanel, c);
		configPanel.add(Box.createVerticalStrut(5), c);
		c.fill = GridBagConstraints.NONE;
		JPanel formatPanel = new JPanel();
		formatPanel.setLayout(new BoxLayout(formatPanel, BoxLayout.X_AXIS));
		formatPanel.add(outputCombo);
		formatPanel.add(exportButton);
		configPanel.add(formatPanel, c);
		configPanel.add(Box.createVerticalStrut(5), c);

		c.fill = GridBagConstraints.BOTH;
		configPanel.add(new JLabel("<HTML><B>SELECTED HEURISTICS</B></HTML>"), c);
		addSpacer(configPanel, c);
		configPanel.add(Box.createVerticalStrut(5), c);
		c.fill = GridBagConstraints.NONE;
		JPanel heuristicsPanel = new JPanel();
		heuristicsPanel.setLayout(new GridBagLayout());
		GridBagConstraints c1 = new GridBagConstraints();
		c1.anchor = GridBagConstraints.WEST;
		c1.ipadx = 15;
		c1.gridx = 0;
		dependencyHeuristicLabel = new JLabel("Dependency Heuristic");
		heuristicsPanel.add(dependencyHeuristicLabel, c1);
		c1.gridx = 1;
		heuristicsPanel.add(dependencyHeuristic, c1);
		c1.gridx = 0;
		dataDependencyHeuristicLabel = new JLabel("Conditional Heuristic");
		heuristicsPanel.add(dataDependencyHeuristicLabel, c1);
		c1.gridx = 1;
		heuristicsPanel.add(dataDependencyHeuristic, c1);
		c1.gridx = 0;
		bindingsHeuristicLabel = new JLabel("Bindings Heuristic");
		heuristicsPanel.add(bindingsHeuristicLabel, c1);
		c1.gridx = 1;
		heuristicsPanel.add(bindingHeuristic, c1);
		c1.gridx = 0;
		decisionHeuristicLabel = new JLabel("Decision Heuristic");
		heuristicsPanel.add(decisionHeuristicLabel, c1);
		c1.gridx = 1;
		heuristicsPanel.add(decisionHeuristic, c1);
		c1.gridx = 0;
		conformanceHeuristicLabel = new JLabel("Conformance Heuristic");
		heuristicsPanel.add(conformanceHeuristicLabel, c1);
		c1.gridx = 1;
		heuristicsPanel.add(conformanceHeuristic, c1);
		configPanel.add(heuristicsPanel, c);
		configPanel.add(Box.createVerticalStrut(5), c);

		c.fill = GridBagConstraints.BOTH;
		configPanel.add(new JLabel("<HTML><B>OPTIONS & THRESHOLDS</B></HTML>"), c);
		addSpacer(configPanel, c);
		configPanel.add(Box.createVerticalStrut(5), c);
		c.weighty = 1;
		configPanel.add(sliderPanel, c);
		c.weighty = 0;
		thresholdResultLabel = new JLabel(
				MessageFormat.format("Using {0}/{1} directly-follows relations occuring at least {2} times",
						computeUsedDirectlyFollows(eventStorage, frequencySlider.getValue()),
						eventStorage.getDirectlyFollowsRelations().elementSet().size(),
						computeObservationCount(eventStorage, frequencySlider.getValue())));
		configPanel.add(Box.createVerticalStrut(5), c);
		configPanel.add(thresholdResultLabel, c);
		configPanel.add(Box.createVerticalStrut(10), c);
		configPanel.add(checkboxPanel, c);
		configPanel.add(Box.createVerticalStrut(5), c);

		configPanel.add(new JLabel("<HTML><B>LEGEND</B></HTML>"), c);
		addSpacer(configPanel, c);
		configPanel.add(Box.createVerticalStrut(5), c);
		ColorSchemeLegend colorLegend = new ColorSchemeLegend(
				Lists.asList(Color.LIGHT_GRAY, ColorScheme.BLUE_SINGLE_HUE.getColors()).toArray(new Color[0])) {

			private static final long serialVersionUID = -7445917701981037923L;

			protected String getBarDesc(int bucket) {
				float adjustedBucketSize = 1.0f / ColorScheme.BLUE_SINGLE_HUE.getColors().length;
				float bucketStart = (bucket - 1) * adjustedBucketSize;
				return MessageFormat.format("{0,number,#.#}", bucketStart);
			}

			protected String getBucketDesc(int bucket) {
				if (bucket == 1) {
					return "none";
				} else {
					return null;
				}
			}

		};
		colorLegend.setMinimumSize(new Dimension(200, 30));
		colorLegend.setPreferredSize(new Dimension(200, 30));
		colorLegend.setMaximumSize(new Dimension(200, 40));
		configPanel.add(colorLegend, c);
		configPanel.add(Box.createVerticalStrut(5), c);

		alignmentLegend = new JPanel();
		alignmentLegend.setLayout(new BoxLayout(alignmentLegend, BoxLayout.X_AXIS));

		JPanel logPanel = new JPanel();
		logPanel.setBackground(DataAwareStepTypes.L.getColor());
		logPanel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		JLabel logLabel = new JLabel("log deviation");
		logLabel.setFont(logLabel.getFont().deriveFont(8.0f));
		logPanel.add(logLabel);
		alignmentLegend.add(logPanel);
		
		JPanel modelPanel = new JPanel();
		modelPanel.setBackground(DataAwareStepTypes.MREAL.getColor());
		modelPanel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		JLabel modelLabel = new JLabel("model deviation");
		modelLabel.setFont(modelLabel.getFont().deriveFont(8.0f));
		modelPanel.add(modelLabel);
		alignmentLegend.add(modelPanel);

		JPanel lmGoodPanel = new JPanel();
		lmGoodPanel.setBackground(DataAwareStepTypes.LMGOOD.getColor());
		lmGoodPanel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		JLabel noLabel = new JLabel("no deviation");
		noLabel.setFont(noLabel.getFont().deriveFont(8.0f));
		noLabel.setForeground(Color.WHITE);
		lmGoodPanel.add(noLabel);
		alignmentLegend.add(lmGoodPanel);
		configPanel.add(alignmentLegend, c);

		add(configPanel, BorderLayout.EAST);
		update();
	}

	private static void createAttributeHelper(final DataRelationStorage dataStorage, final JTextField searchQuery) {
		final JPopupMenu menu = new JPopupMenu();
		ActionListener itemListener = new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				AttributeMenuItem item = (AttributeMenuItem) e.getSource();
				String operator = "=";
				searchQuery.setText(searchQuery.getText() + " " + item.getQuery(operator));
			}

		};
		for (String key : dataStorage.getAvailableAttributes().keySet()) {
			Type type = dataStorage.getAttributeTypes().get(WekaUtil.fixVarName(key));
			if (type == Type.BOOLEAN) {
				JMenu subMenu = new JMenu(key + " (" + type + ")");
				subMenu.add(new BooleanAttributeMenuItem(key, "true")).addActionListener(itemListener);
				subMenu.add(new BooleanAttributeMenuItem(key, "false")).addActionListener(itemListener);
				menu.add(subMenu);
			} else if (type == Type.LITERAL) {
				JMenu subMenu = new JMenu(key + " (" + type + ")");
				Set<String> literals = dataStorage.getLiteralValues().get(WekaUtil.fixVarName(key));
				for (String value : Iterables.limit(literals, 20)) {
					subMenu.add(new AttributeMenuItem(key, value)).addActionListener(itemListener);
				}
				if (literals.size() > 20) {
					subMenu.add(new JMenuItem("... and " + (literals.size() - 20) + " more"));
				}
				menu.add(subMenu).addActionListener(itemListener);
			} else {
				menu.add(new OtherAttributeMenuItem(key + " (" + type + ")", key)).addActionListener(itemListener);
			}
		}
		searchQuery.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if (e.isPopupTrigger())
					showMenu(e);
			}

			public void mouseReleased(MouseEvent e) {
				if (e.isPopupTrigger())
					showMenu(e);
			}

			private void showMenu(MouseEvent e) {
				menu.show(e.getComponent(), e.getX(), e.getY());
			}
		});
	}

	private static int computeObservationCount(EventRelationStorage eventStorage, double frequencyThreshold) {
		int traces = eventStorage.countTraces();
		return (int) Math.ceil(frequencyThreshold * traces);
	}

	private static int computeUsedDirectlyFollows(EventRelationStorage eventStorage, double frequencyThreshold) {
		int minimumCount = computeObservationCount(eventStorage, frequencyThreshold);
		int usedDF = 0;
		for (Entry<Relation> entry : eventStorage.getDirectlyFollowsRelations().entrySet()) {
			if (entry.getCount() >= minimumCount) {
				usedDF++;
			}
		}
		return usedDF;
	}

	private static DefaultListCellRenderer createHeuristicExtensionRenderer() {
		return new DefaultListCellRenderer() {

			private static final long serialVersionUID = 1L;

			public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected,
					boolean cellHasFocus) {
				if (value != null) {
					HeuristicExtension heuristicExtension = (HeuristicExtension) value;
					Component component = super.getListCellRendererComponent(list, heuristicExtension.getName(), index,
							isSelected, cellHasFocus);
					if (heuristicExtension.getReference() != null) {
						if (component instanceof JComponent) {
							((JComponent) component).setToolTipText(heuristicExtension.getReference());
						}
					}
					return component;
				} else {
					return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				}
			}

		};
	}

	private static void addSpacer(JPanel panel, GridBagConstraints c) {
		JPanel spacer1 = new JPanel(new BorderLayout());
		spacer1.setBorder(createConfigPanelBorder());
		panel.add(spacer1, c);
	}

	private void selectMethod(Class<?> miningMethod) {
		for (int i = 0; i < outputCombo.getModel().getSize(); i++) {
			if (outputCombo.getModel().getElementAt(i).getClass() == miningMethod) {
				outputCombo.setSelectedIndex(i);
			}
		}
	}

	private static CompoundBorder createConfigPanelBorder() {
		return BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(2, 0, 0, 0, Color.BLACK),
				BorderFactory.createEmptyBorder(2, 0, 0, 0));
	}

	public void update() {
		if (isUpdating) {
			return; // wait 
		}
		beforeUpdate();
		final MiningMethod miningMethod = getCurrentMiningMethod();
		updateThresholds(miningMethod.getConfiguration());
		final boolean needsDataUpdate = updateConfiguration(miningMethod);
		updateConfigUI(miningMethod);

		new SwingWorker<JComponent, Void>() {

			protected JComponent doInBackground() throws Exception {
				final ExecutorService executor = Executors
						.newFixedThreadPool(Runtime.getRuntime().availableProcessors());
				try {
					MinerContext minerContext = createMiningContext(executor);
					Stopwatch watch = Stopwatch.createStarted();
					if (needsDataUpdate) {
						filterLog(eventStorage.getClassifier(), dataStorage.getAvailableAttributes());
						miningMethod.updateData(eventStorage, dataStorage);
					}
					JComponent result = miningMethod.mine(minerContext);
					watch.stop();
					System.out.println(MessageFormat.format("Visualisation updated in {0,number,#}ms",
							watch.elapsed(TimeUnit.MILLISECONDS)));
					return result;
				} finally {
					executor.shutdown();
				}
			}

			private void filterLog(XEventClassifier classifier, ImmutableMap<String, Class<?>> availableAttributes)
					throws ParseException {
				if (searchQuery.getText().isEmpty()) {
					if (eventStorage.getAllTraces().size() != originalTraces.size()) {
						filterStorage(classifier, availableAttributes, originalTraces);
					} // else ignore
				} else {
					filterStorage(classifier, availableAttributes,
							ImmutableList.copyOf(filterTraces(searchQuery.getText())));
				}
			}

			private void filterStorage(XEventClassifier classifier, ImmutableMap<String, Class<?>> availableAttributes,
					Collection<XTrace> filteredTraces) {
				ExecutorService executorService = Executors
						.newFixedThreadPool(Runtime.getRuntime().availableProcessors());
				try {
					eventStorage = EventRelationStorage.Factory.createEventRelations(filteredTraces, classifier,
							executorService);
					ImmutableSet<String> currentConsideredAttributes = dataStorage.getConsideredAttributes();
					dataStorage = DataRelationStorage.Factory.createDataRelations(eventStorage, availableAttributes,
							currentConsideredAttributes, executorService);
				} finally {
					executorService.shutdown();
				}
			}

			private Iterable<XTrace> filterTraces(String query) throws ParseException {
				String approximateQuery = addApproximateMatching(searchQuery.getText().trim());
				Iterable<XTrace> traceResult = XIndex.filterTracesOnAttributesOrNames(originalTraces, approximateQuery,
						new AttributeTypeResolverNaiveImpl(originalTraces));
				return traceResult;
			}

			private String addApproximateMatching(String query) {
				if (!query.isEmpty() && !(query.startsWith("%") || (query.contains("=") || query.contains(">")
						|| query.contains("<") || query.contains("%") || query.contains("~")))) {
					query = "%" + query;
				}
				return query;
			}

			protected void done() {
				try {
					wrapper.removeAll();
					wrapper.add(get());
					updateStatistics();
				} catch (InterruptedException | ExecutionException e) {
					ProMUIHelper.showErrorMessage(wrapper, e.getMessage(), "Error updating", e);
				} finally {
					afterUpdate();
				}
			}

			private void updateStatistics() {
				searchResultLabel.setText(
						MessageFormat.format("Using {0}/{1} traces, {2}/{3} events", eventStorage.getAllTraces().size(),
								originalTraces.size(), eventStorage.countEvents(), originalEventCount));
				thresholdResultLabel.setText(
						MessageFormat.format("Using {0}/{1} directly-follows relations occuring at least {2} times",
								computeUsedDirectlyFollows(eventStorage, frequencySlider.getValue()),
								eventStorage.getDirectlyFollowsRelations().elementSet().size(),
								computeObservationCount(eventStorage, frequencySlider.getValue())));
			}

		}.execute();
	}

	private MinerContext createMiningContext(final ExecutorService executor) {
		final MinerContext minerContext = new MinerContext() {

			public Progress getProgress() {
				return progressListener;
			}

			public ExecutorService getExecutor() {
				return executor;
			}

			public PluginContext getPluginContext() {
				return controller.getContext();
			}

		};
		return minerContext;
	}

	private void beforeUpdate() {
		isUpdating = true;
		progressBar.setString("processing ...");
		progressBar.setIndeterminate(true);
		wrapper.removeAll();
		wrapper.add(new JPanel(), BorderLayout.CENTER);
		JPanel progressPanel = new JPanel(new BorderLayout());
		progressPanel.add(progressBar, BorderLayout.CENTER);
		progressPanel.add(cancelButton, BorderLayout.EAST);
		wrapper.add(progressPanel, BorderLayout.SOUTH);
		wrapper.validate();
		recursiveEnableControls(configPanel, false);
		recursiveEnableControls(dataConfigurationPanel, false);
	}

	private void afterUpdate() {
		recursiveEnableControls(configPanel, true);
		recursiveEnableControls(dataConfigurationPanel, true);
		revalidate();
		isUpdating = false;
	}

	private void recursiveEnableControls(Container panel, boolean enabled) {
		for (Component cmp : panel.getComponents()) {
			cmp.setEnabled(enabled);
			if (cmp instanceof Container) {
				recursiveEnableControls((Container) cmp, enabled);
			}
		}
	}

	private void updateConfigUI(MiningMethod method) {
		MethodConfiguration c = method.getConfiguration();
		dependencySlider.setVisible(c.usesDependencyThreshold());
		if (expertCheckbox.isSelected() || !c.usesDependencyThreshold()) {
			l1Slider.setVisible(c.usesL1Threshold());
			l2Slider.setVisible(c.usesL2Threshold());
			relativeToBestSlider.setVisible(c.usesRelativeToBestThreshold());
		} else {
			l1Slider.setVisible(false);
			l2Slider.setVisible(false);
			relativeToBestSlider.setVisible(false);
		}
		frequencySlider.setVisible(c.usesFrequencyThreshold());
		longTermCheckbox.setVisible(c.usesLongTermConnected());
		allConnectedCheckbox.setVisible(c.usesAllTaskConnected());
		acceptedConnectedCheckbox.setVisible(c.usesAcceptedTasksConnected());
		noUnconnectedCheckbox.setVisible(c.usesNoTaskConnected());
		bindingSlider.setVisible(c.usesBindingsThreshold());
		bindingHeuristic.setVisible(c.usesBindingsThreshold());
		conditionSlider.setVisible(c.usesDataThreshold());
		configureClassification.setVisible(c.usesClassification());
		decisionSlider.setVisible(c.usesDecisionThreshold());

		dependencyHeuristic.setVisible(c.usesDependencyThreshold());
		dependencyHeuristicLabel.setVisible(c.usesDependencyThreshold());
		bindingHeuristic.setVisible(c.usesBindingsThreshold());
		bindingsHeuristicLabel.setVisible(c.usesBindingsThreshold());
		dataDependencyHeuristic.setVisible(c.usesDataThreshold());
		dataDependencyHeuristicLabel.setVisible(c.usesDataThreshold());
		decisionHeuristic.setVisible(c.usesDecisionThreshold());
		decisionHeuristicLabel.setVisible(c.usesDecisionThreshold());

		conformanceHeuristic.setVisible(c.usesConformance());
		conformanceHeuristicLabel.setVisible(c.usesConformance());
		alignmentLegend.setVisible(conformanceHeuristic.getSelectedItem() instanceof ConformanceExtensionAlignmentImpl);
	}

	public MiningMethod getCurrentMiningMethod() {
		return (MiningMethod) outputCombo.getSelectedItem();
	}

	/**
	 * @param miningMethod
	 * @return needs updated data structures
	 */
	public boolean updateConfiguration(final MiningMethod miningMethod) {
		boolean needsDataUpdate = false;
		needsDataUpdate |= updateDataDiscovery(miningMethod.getConfiguration());
		needsDataUpdate |= updateExtensions(miningMethod.getConfiguration());
		needsDataUpdate |= updateFilter(miningMethod.getConfiguration());
		return needsDataUpdate;
	}

	/**
	 * @param c
	 *            old configuration to update
	 * @return needs updated data structures
	 */
	public void updateThresholds(MethodConfiguration c) {
		c.setDependencyThreshold(dependencySlider.getValue());
		c.setL1Threshold(l1Slider.getValue());
		c.setL2Threshold(l2Slider.getValue());
		c.setRelativeToBestThreshold(relativeToBestSlider.getValue());
		c.setFrequencyThreshold(frequencySlider.getValue());
		c.setLongTerm(longTermCheckbox.isSelected());
		c.setAllTasksConnected(allConnectedCheckbox.isSelected());
		c.setBindingsThreshold(bindingSlider.getValue());
		c.setConditionThreshold(conditionSlider.getValue());
		c.setAcceptedTasksConnected(acceptedConnectedCheckbox.isSelected());
		c.setNoTasksUnconnected(noUnconnectedCheckbox.isSelected());
		c.setDecisionThreshold(decisionSlider.getValue());
	}

	/**
	 * @param c
	 *            old configuration to update
	 * @return needs updated data structures
	 */
	public boolean updateExtensions(MethodConfiguration c) {
		boolean hasChanged = false;
		BindingsHeuristicExtension newBindingsHeuristic = (BindingsHeuristicExtension) bindingHeuristic
				.getSelectedItem();
		if (!newBindingsHeuristic.equals(c.getBindingsHeuristicExtension())) {
			c.setBindingsHeuristicExtension(newBindingsHeuristic);
			hasChanged = true;
		}
		DependencyHeuristicExtension dependencyHeuristicFactory = (DependencyHeuristicExtension) dependencyHeuristic
				.getSelectedItem();
		if (!dependencyHeuristicFactory.equals(c.getDependencyHeuristicExtension())) {
			c.setDependencyHeuristicExtension(dependencyHeuristicFactory);
			hasChanged = true;
		}
		ConformanceHeuristicExtension conformanceHeuristicExtension = (ConformanceHeuristicExtension) conformanceHeuristic
				.getSelectedItem();
		if (!conformanceHeuristicExtension.equals(c.getConformanceHeuristicExtension())) {
			c.setConformanceHeuristicExtension(conformanceHeuristicExtension);
			hasChanged = true;
		}
		ConditionalDependencyHeuristicExtension dataDependencyHeuristicExtension = (ConditionalDependencyHeuristicExtension) dataDependencyHeuristic
				.getSelectedItem();
		if (!dataDependencyHeuristicExtension.equals(c.getDataDependencyHeuristicExtension())) {
			c.setDataDependencyHeuristicExtension(dataDependencyHeuristicExtension);
			hasChanged = true;
		}
		DecisionHeuristicExtension decisionHeuristicExtension = (DecisionHeuristicExtension) decisionHeuristic
				.getSelectedItem();
		if (!decisionHeuristicExtension.equals(c.getDecisionHeuristicExtension())) {
			c.setDecisionHeuristicExtension(decisionHeuristicExtension);
			hasChanged = true;
		}		
		return hasChanged;
	}

	/**
	 * @param c
	 *            old configuration to update
	 * @return needs updated data structures
	 */
	public boolean updateDataDiscovery(MethodConfiguration c) {
		c.getDataDiscoveryConfig()
				.setMinPercentageObjectsOnLeaf(dataConfigurationPanel.getMinPercentageObjectsOnLeaf());
		c.getDataDiscoveryConfig().setCrossValidate(dataConfigurationPanel.isCrossCalidate());
		c.getDataDiscoveryConfig().setBinarySplit(dataConfigurationPanel.isBinarySplit());
		if (!dataConfigurationPanel.getSelectedAttributes().equals(c.getDataDiscoveryAttributes())) {
			c.setDataDiscoveryAttributes(dataConfigurationPanel.getSelectedAttributes());
			return true;
		}
		return false;
	}

	/**
	 * @param c
	 *            old configuration to update
	 * @return needs updated data structures
	 */
	public boolean updateFilter(MethodConfiguration c) {
		if (!searchQuery.getText().equals(c.getFilterQuery())) {
			c.setFilterQuery(searchQuery.getText());
			return true;
		}
		return false;
	}

}