package org.processmining.dataawarecnetminer.interactive.methods;

import java.text.MessageFormat;
import java.text.NumberFormat;

import org.processmining.dataawarecnetminer.extension.bindings.BindingsHeuristic;
import org.processmining.dataawarecnetminer.extension.conditionaldependencies.ConditionalDependencyHeuristic;
import org.processmining.dataawarecnetminer.extension.conformance.ConformanceHeuristic;
import org.processmining.dataawarecnetminer.extension.decision.DecisionHeuristic;
import org.processmining.dataawarecnetminer.extension.dependencies.DependencyHeuristic;
import org.processmining.dataawarecnetminer.model.DataRelationStorage;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;

abstract public class AbstractMiningMethod implements MiningMethod {

	private MethodConfiguration config = new MethodConfiguration();

	private EventRelationStorage eventStorage;
	private DataRelationStorage dataStorage;

	private DependencyHeuristic dependencyHeuristic;
	private BindingsHeuristic bindingsHeuristic;
	private ConditionalDependencyHeuristic dataHeuristic;
	private ConformanceHeuristic conformanceHeuristic;
	private DecisionHeuristic decisionHeuristic;

	public AbstractMiningMethod() {
		super();
	}

	public MethodConfiguration getConfiguration() {
		return config;
	}

	protected String formatInteger(int count) {
		return NumberFormat.getIntegerInstance().format(count);
	}

	protected String formatDouble(double dependencyValue) {
		return MessageFormat.format("{0,number,#.###}", dependencyValue);
	}

	public EventRelationStorage getEventStorage() {
		if (eventStorage == null) {
			throw new IllegalStateException("Please set event storage");
		}
		return eventStorage;
	}

	public DataRelationStorage getDataStorage() {
		if (dataStorage == null) {
			throw new IllegalStateException("Please set event storage");
		}
		return dataStorage;
	}

	public void updateData(EventRelationStorage eventStorage, DataRelationStorage dataStorage) {
		this.eventStorage = eventStorage;
		this.dataStorage = dataStorage;
		this.dependencyHeuristic = config.getDependencyHeuristicExtension().newInstance(eventStorage);
		this.bindingsHeuristic = config.getBindingsHeuristicExtension().newInstance(eventStorage);
		this.dataHeuristic = config.getDataDependencyHeuristicExtension().newInstance(eventStorage, dataStorage);
		this.decisionHeuristic = config.getDecisionHeuristicExtension().newInstance(eventStorage, dataStorage);
		this.conformanceHeuristic = config.getConformanceHeuristicExtension().newInstance(eventStorage);
	}

	public DependencyHeuristic getDependencyHeuristic() {
		return dependencyHeuristic;
	}

	public BindingsHeuristic getBindingsHeuristic() {
		return bindingsHeuristic;
	}

	public ConditionalDependencyHeuristic getConditionalHeuristic() {
		return dataHeuristic;
	}

	public DecisionHeuristic getDecisionHeuristic() {
		return decisionHeuristic;
	}

	public ConformanceHeuristic getConformanceHeuristic() {
		return conformanceHeuristic;
	}

}