package org.processmining.dataawarecnetminer.interactive.methods;

import java.util.HashSet;
import java.util.Set;

import org.processmining.dataawarecnetminer.extension.bindings.BindingsHeuristicExtension;
import org.processmining.dataawarecnetminer.extension.bindings.BindingsHeuristicExtensionNearestActivityImpl;
import org.processmining.dataawarecnetminer.extension.conditionaldependencies.ConditionalDependencyHeuristicExtension;
import org.processmining.dataawarecnetminer.extension.conditionaldependencies.ConditionalDependencyHeuristicExtensionC45KappaImpl;
import org.processmining.dataawarecnetminer.extension.conformance.ConformanceExtensionPassthroughImpl;
import org.processmining.dataawarecnetminer.extension.conformance.ConformanceHeuristicExtension;
import org.processmining.dataawarecnetminer.extension.decision.DecisionHeuristicExtension;
import org.processmining.dataawarecnetminer.extension.decision.DecisionHeuristicExtensionC45Impl;
import org.processmining.dataawarecnetminer.extension.dependencies.DependencyHeuristicExtension;
import org.processmining.dataawarecnetminer.extension.dependencies.DependencyHeuristicExtensionFHMImpl;
import org.processmining.datadiscovery.DecisionTreeConfig;

public class MethodConfiguration {

	private String filterQuery;

	private double frequencyThreshold = 0.1;
	private double dependencyThreshold = 0.9;
	private double l1Threshold = 0.9;
	private double l2Threshold = 0.9;
	private double relativeToBestThreshold = 0.00;
	private DependencyHeuristicExtension dependencyHeuristicExtension = new DependencyHeuristicExtensionFHMImpl();

	// Bindings
	private double bindingsThreshold = 0.1;
	private BindingsHeuristicExtension bindingsHeuristicExtension = new BindingsHeuristicExtensionNearestActivityImpl();

	// Frequencies
	private ConformanceHeuristicExtension conformanceHeuristicExtension = new ConformanceExtensionPassthroughImpl();

	// Data addition
	private ConditionalDependencyHeuristicExtension dataDependencyHeuristicExtension = new ConditionalDependencyHeuristicExtensionC45KappaImpl();
	private double conditionThreshold = 0.5;
	private double minInstancesOnLeaf = 0.01;
	private DecisionTreeConfig decisionTreeConfig = new DecisionTreeConfig();
	private Set<String> dataDiscoveryAttributes = new HashSet<>();

	// Decision mining
	private DecisionHeuristicExtension decisionHeuristicExtension = new DecisionHeuristicExtensionC45Impl();
	private double decisionThreshold = 0.8;
	
	private boolean allTasksConnected = true;
	private boolean acceptedTasksConnected = false;
	private boolean noTasksUnconnected = false;
	private boolean longTerm = false;

	private boolean usesData;
	private boolean usesDependency;
	private boolean usesL1;
	private boolean usesL2;
	private boolean usesRelativeToBest;
	private boolean usesFrequency;
	private boolean usesLongTerm;
	private boolean usesAllTask;
	private boolean usesAcceptedTasks;
	private boolean usesNoTask;
	private boolean usesBindings;
	private boolean usesClassification;
	private boolean usesConformance;
	private boolean usesDecision;

	public MethodConfiguration() {
	}

	public double getFrequencyThreshold() {
		return frequencyThreshold;
	}

	public MethodConfiguration setFrequencyThreshold(double frequencyThreshold) {
		this.frequencyThreshold = frequencyThreshold;
		return this;
	}

	public double getDependencyThreshold() {
		return dependencyThreshold;
	}

	public MethodConfiguration setDependencyThreshold(double dependencyThreshold) {
		this.dependencyThreshold = dependencyThreshold;
		return this;
	}

	public double getConditionThreshold() {
		return conditionThreshold;
	}

	public MethodConfiguration setConditionThreshold(double conditionThreshold) {
		this.conditionThreshold = conditionThreshold;
		return this;
	}

	public double getMinInstancesOnLeaf() {
		return minInstancesOnLeaf;
	}

	public MethodConfiguration setMinInstancesOnLeaf(double minInstancesOnLeaf) {
		this.minInstancesOnLeaf = minInstancesOnLeaf;
		return this;
	}

	public double getBindingsThreshold() {
		return bindingsThreshold;
	}

	public MethodConfiguration setBindingsThreshold(double bindingsThreshold) {
		this.bindingsThreshold = bindingsThreshold;
		return this;
	}

	public boolean isAllTasksConnected() {
		return allTasksConnected;
	}

	public MethodConfiguration setAllTasksConnected(boolean allTasksConnected) {
		this.allTasksConnected = allTasksConnected;
		return this;
	}

	public boolean isAcceptedTasksConnected() {
		return acceptedTasksConnected;
	}

	public MethodConfiguration setAcceptedTasksConnected(boolean acceptedTasksConnected) {
		this.acceptedTasksConnected = acceptedTasksConnected;
		return this;
	}

	public boolean isNoTasksUnconnected() {
		return noTasksUnconnected;
	}

	public MethodConfiguration setNoTasksUnconnected(boolean noTasksUnconnected) {
		this.noTasksUnconnected = noTasksUnconnected;
		return this;
	}

	public boolean isLongTerm() {
		return longTerm;
	}

	public MethodConfiguration setLongTerm(boolean longTerm) {
		this.longTerm = longTerm;
		return this;
	}

	public boolean usesDataThreshold() {
		return usesData;
	}

	public boolean usesDependencyThreshold() {
		return usesDependency;
	}

	public boolean usesFrequencyThreshold() {
		return usesFrequency;
	}

	public boolean usesLongTermConnected() {
		return usesLongTerm;
	}

	public boolean usesAllTaskConnected() {
		return usesAllTask;
	}

	public boolean usesAcceptedTasksConnected() {
		return usesAcceptedTasks;
	}

	public boolean usesNoTaskConnected() {
		return usesNoTask;
	}

	public boolean usesBindingsThreshold() {
		return usesBindings;
	}

	public boolean usesClassification() {
		return usesClassification;
	}

	public void setUsesData(boolean usesData) {
		this.usesData = usesData;
	}

	public void setUsesDependency(boolean usesDependency) {
		this.usesDependency = usesDependency;
	}

	public void setUsesFrequency(boolean usesFrequency) {
		this.usesFrequency = usesFrequency;
	}

	public void setUsesLongTerm(boolean usesLongTerm) {
		this.usesLongTerm = usesLongTerm;
	}

	public void setUsesAllTask(boolean usesAllTask) {
		this.usesAllTask = usesAllTask;
	}

	public void setUsesAcceptedTasks(boolean usesAcceptedTasks) {
		this.usesAcceptedTasks = usesAcceptedTasks;
	}

	public void setUsesNoTask(boolean usesNoTask) {
		this.usesNoTask = usesNoTask;
	}

	public void setUsesBindings(boolean usesBindings) {
		this.usesBindings = usesBindings;
	}

	public void setUsesClassification(boolean usesClassification) {
		this.usesClassification = usesClassification;
	}

	public void setUsesConformance(boolean usesConformance) {
		this.usesConformance = usesConformance;
	}
	
	public boolean usesConformance() {
		return usesConformance;
	}

	public double getL1Threshold() {
		return l1Threshold;
	}

	public double getL2Threshold() {
		return l2Threshold;
	}

	public MethodConfiguration setL2Threshold(double l2Threshold) {
		this.l2Threshold = l2Threshold;
		return this;
	}

	public MethodConfiguration setL1Threshold(double l1Threshold) {
		this.l1Threshold = l1Threshold;
		return this;
	}

	public boolean usesL1Threshold() {
		return usesL1;
	}

	public void setUsesL1Threshold(boolean usesL1) {
		this.usesL1 = usesL1;
	}

	public boolean usesL2Threshold() {
		return usesL2;
	}

	public void setUsesL2Threshold(boolean usesL2) {
		this.usesL2 = usesL2;
	}

	public double getRelativeToBestThreshold() {
		return relativeToBestThreshold;
	}

	public MethodConfiguration setRelativeToBestThreshold(double relativeToBestThreshold) {
		this.relativeToBestThreshold = relativeToBestThreshold;
		return this;
	}

	public boolean usesRelativeToBestThreshold() {
		return usesRelativeToBest;
	}

	public void setUsesRelativeToBestThreshold(boolean usesRelativeToBest) {
		this.usesRelativeToBest = usesRelativeToBest;
	}

	public String getFilterQuery() {
		return filterQuery;
	}

	public void setFilterQuery(String filterQuery) {
		this.filterQuery = filterQuery;
	}

	public BindingsHeuristicExtension getBindingsHeuristicExtension() {
		return bindingsHeuristicExtension;
	}

	public void setBindingsHeuristicExtension(BindingsHeuristicExtension bindingsHeuristicFactory) {
		this.bindingsHeuristicExtension = bindingsHeuristicFactory;
	}

	public DependencyHeuristicExtension getDependencyHeuristicExtension() {
		return dependencyHeuristicExtension;
	}

	public void setDependencyHeuristicExtension(DependencyHeuristicExtension dependencyHeuristicFactory) {
		this.dependencyHeuristicExtension = dependencyHeuristicFactory;
	}

	public ConformanceHeuristicExtension getConformanceHeuristicExtension() {
		return conformanceHeuristicExtension;
	}

	public void setConformanceHeuristicExtension(ConformanceHeuristicExtension conformanceHeuristicExtension) {
		this.conformanceHeuristicExtension = conformanceHeuristicExtension;
	}

	public DecisionTreeConfig getDataDiscoveryConfig() {
		return decisionTreeConfig;
	}

	public DecisionTreeConfig getDecisionTreeConfig() {
		return decisionTreeConfig;
	}

	public void setDecisionTreeConfig(DecisionTreeConfig decisionTreeConfig) {
		this.decisionTreeConfig = decisionTreeConfig;
	}

	public ConditionalDependencyHeuristicExtension getDataDependencyHeuristicExtension() {
		return dataDependencyHeuristicExtension;
	}

	public void setDataDependencyHeuristicExtension(ConditionalDependencyHeuristicExtension dataDependencyHeuristicExtension) {
		this.dataDependencyHeuristicExtension = dataDependencyHeuristicExtension;
	}

	public void setDataDiscoveryAttributes(Set<String> selectedAttributes) {
		this.dataDiscoveryAttributes = selectedAttributes;
	}

	public Set<String> getDataDiscoveryAttributes() {
		return dataDiscoveryAttributes;
	}

	public DecisionHeuristicExtension getDecisionHeuristicExtension() {
		return decisionHeuristicExtension;
	}

	public void setDecisionHeuristicExtension(DecisionHeuristicExtension decisionHeuristicExtension) {
		this.decisionHeuristicExtension = decisionHeuristicExtension;
	}

	public double getDecisionThreshold() {
		return decisionThreshold;
	}

	public void setDecisionThreshold(double decisionThreshold) {
		this.decisionThreshold = decisionThreshold;
	}

	public boolean usesDecisionThreshold() {
		return usesDecision;
	}

	public MethodConfiguration setUsesDecision(boolean usesDecision) {
		this.usesDecision = usesDecision;
		return this;
	}

}