package org.processmining.dataawarecnetminer.interactive.methods.cnet;

import javax.swing.JComponent;

import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.mining.classic.HeuristicsCausalNetMiner;
import org.processmining.dataawarecnetminer.model.DataAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.dataawarecnetminer.model.FrequencyAwareCausalNet;
import org.processmining.dataawarecnetminer.visualization.DataScoreEdgeDecorator;
import org.processmining.dataawarecnetminer.visualization.FrequencyNodeDecorator;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.cnet.CausalNet;
import org.processmining.plugins.dcnet.DataAwareCausalNetVisualization;
import org.processmining.plugins.utils.ProvidedObjectHelper;

public class DataAwareCNetMethod extends DataDependencyGraphMethod {

	private FrequencyAwareCausalNet causalNet;

	public DataAwareCNetMethod() {
		super();
		getConfiguration().setUsesLongTerm(true);
		getConfiguration().setUsesNoTask(true);
		getConfiguration().setUsesAcceptedTasks(true);
		getConfiguration().setUsesBindings(true);
		getConfiguration().setUsesConformance(true);
	}

	@Override
	public JComponent mine(MinerContext context) throws MiningException {
		context.getProgress().setCaption("discover dependencies ...");
		DataAwareCausalGraph dependencyGraph = buildDataDependencyGraph(context);

		context.getProgress().setCaption("determine bindings ...");
		FrequencyAwareCausalNet initialCausalNet = mineCNet(context, getEventStorage(), dependencyGraph);

		context.getProgress().setCaption("compute conformance information (this might take a long time) ...");
		causalNet = getConformanceHeuristic().computeFrequencies(context, initialCausalNet);

		DataScoreEdgeDecorator edgeDecorator = new DataScoreEdgeDecorator(causalNet, dependencyGraph, getEventStorage());
		FrequencyNodeDecorator nodeDecorator = new FrequencyNodeDecorator(causalNet);
		DataAwareCausalNetVisualization netVisualization = new DataAwareCausalNetVisualization(causalNet.getCNet(),
				nodeDecorator, edgeDecorator);

		return netVisualization.getVisualization();
	}

	@Override
	public void save(PluginContext context, String fileName) throws MiningException {
		ProvidedObjectHelper.publish(context, fileName, causalNet.getCNet(), CausalNet.class, false);
		ProvidedObjectHelper.setFavorite(context, causalNet.getCNet());
	}

	protected FrequencyAwareCausalNet mineCNet(MinerContext context, final EventRelationStorage eventStorage,
			DataAwareCausalGraph dependencyGraph) throws MiningException {

		HeuristicsCausalNetMiner causalNetMiner = new HeuristicsCausalNetMiner(eventStorage);
		causalNetMiner.getConfig().setBindingsThreshold(getConfiguration().getBindingsThreshold());
		causalNetMiner.getConfig().setConsiderLongDistanceRelations(getConfiguration().isLongTerm());
		causalNetMiner.getConfig().setLongTermDependencyThreshold(getConfiguration().getDependencyThreshold());
		causalNetMiner.getConfig().setNoTasksUnconnected(getConfiguration().isNoTasksUnconnected());
		FrequencyAwareCausalNet causalNet = causalNetMiner.mineCausalNet(context, dependencyGraph,
				getBindingsHeuristic());

		return causalNet;
	}

	public String toString() {
		return "Causal net (C-net)";
	}

}