package org.processmining.dataawarecnetminer.interactive.methods.cnet;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JComponent;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.dataawarecnetminer.common.MinerContext;
import org.processmining.dataawarecnetminer.exception.MiningException;
import org.processmining.dataawarecnetminer.extension.conditionaldependencies.ConditionalDependencyHeuristicExtensionNoneImpl;
import org.processmining.dataawarecnetminer.interactive.util.DotHelper;
import org.processmining.dataawarecnetminer.model.DataAwareCausalGraph;
import org.processmining.dataawarecnetminer.model.DependencyAwareCausalGraph;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.causalgraph.Relation;
import org.processmining.models.causalgraph.SimpleCausalGraph;
import org.processmining.plugins.graphviz.dot.Dot;
import org.processmining.plugins.graphviz.dot.DotEdge;
import org.processmining.plugins.graphviz.dot.DotNode;
import org.processmining.plugins.utils.ProvidedObjectHelper;

public class DataDependencyGraphMethod extends DependencyGraphMethod {

	private DataAwareCausalGraph dependencyGraph;

	public DataDependencyGraphMethod() {
		super();
		getConfiguration().setUsesData(true);
		getConfiguration().setUsesClassification(true);
	}

	@Override
	public JComponent mine(MinerContext context) throws MiningException {
		DotHelper dotHelper = new DotHelper();

		dependencyGraph = buildDataDependencyGraph(context);

		Dot dot = new Dot();
		Map<XEventClass, DotNode> activityToNode = new HashMap<XEventClass, DotNode>();
		for (Relation relation : dependencyGraph.getCausalRelations()) {
			DotEdge dependency = dotHelper.addDependency(getEventStorage(), dot, activityToNode, relation);
			double dependencyValue = dependencyGraph.getDependencyRelation(relation).getDependencyValue();
			dependency.setLabel(MessageFormat.format("{0}", formatDouble(dependencyValue)));
			if (dependencyGraph.getDataRelation(relation) != null) {
				dependency.setOption("color", "red");
				dependency.setOption("fillcolor", "red");
			}
		}

		return dotHelper.createPanel(dot);
	}

	@Override
	public void save(PluginContext context, String fileName) throws MiningException {
		ProvidedObjectHelper.publish(context, fileName, dependencyGraph, SimpleCausalGraph.class, false);
		ProvidedObjectHelper.setFavorite(context, dependencyGraph);
	}

	protected DataAwareCausalGraph buildDataDependencyGraph(MinerContext context) throws MiningException {

		//TODO this is a hack to disable post processing when conditional discovery follows
		//     introducing a proper post processor would be better
		boolean acceptedTasksConnected = getConfiguration().isAcceptedTasksConnected();
		boolean allTasksConnected = getConfiguration().isAllTasksConnected();
		if (!getConfiguration().getDataDependencyHeuristicExtension().getClass().isAssignableFrom(ConditionalDependencyHeuristicExtensionNoneImpl.class)) {
			getConfiguration().setAcceptedTasksConnected(false); // we connect the tasks afterwards
			getConfiguration().setAllTasksConnected(false); // we connect the tasks afterwards			
		}

		DependencyAwareCausalGraph causalGraph = getDependencyHeuristic().computeDependencies(context,
				getConfiguration());

		getConfiguration().setAcceptedTasksConnected(acceptedTasksConnected);
		getConfiguration().setAllTasksConnected(allTasksConnected);

		return getConditionalHeuristic().computeDependencies(context, getConfiguration(), causalGraph);
	}

	public String toString() {
		return "Dependency Graph";
	}

}