package org.processmining.dataawarecnetminer.mining.classic;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.dataawarecnetminer.model.DependencyRelation;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.models.causalgraph.Relation;

public class AllTaskConnectedHeuristic {

	public static void connectAllTasks(EventRelationStorage eventStore,
			final Map<Relation, DependencyRelation> graphRelations, double relativeToBestRatio) {

		Set<Relation> strongestCauses = new HashSet<>();
		Set<Relation> strongestFollowers = new HashSet<>();

		// First connect all unconnected
		for (XEventClass task : eventStore.getEventClasses()) {
			boolean hasSource = task.equals(eventStore.getStartEventClass()); //start does not need source
			boolean hasTarget = task.equals(eventStore.getEndEventClass()); // end does not need target
			for (Relation relation : graphRelations.keySet()) {
				if (!hasSource && !sameClass(relation) && relation.getTarget().equals(task)) {
					hasSource = true; // task has a predecessor
				}
				if (!hasTarget && !sameClass(relation) && relation.getSource().equals(task)) {
					hasTarget = true; // task has a successor
				}
				if (hasSource && hasTarget) {
					break; //stop this task is connected
				}
			}
			if (!hasSource) {
				Relation strongestCause = StrongestRelationsHeuristic.findStrongestCause(eventStore, task);
				strongestCauses.add(strongestCause);
			}
			if (!hasTarget) {
				Relation strongestFollower = StrongestRelationsHeuristic.findStrongestFollower(eventStore, task);
				strongestFollowers.add(strongestFollower);
			}
		}

		for (Relation strongest : strongestCauses) {
			graphRelations.put(strongest, eventStore.getDependencyRelation(strongest));
			StrongestRelationsHeuristic.findRelativeToBestCauses(eventStore, strongest, relativeToBestRatio);
		}
		for (Relation strongest : strongestFollowers) {
			graphRelations.put(strongest, eventStore.getDependencyRelation(strongest));
			StrongestRelationsHeuristic.findRelativeToBestFollowers(eventStore, strongest, relativeToBestRatio);
		}

	}

	private static boolean sameClass(Relation relation) {
		return sameClass(relation.getSource(), relation.getTarget());
	}

	private static boolean sameClass(XEventClass sourceClass, XEventClass targetClass) {
		return sourceClass.equals(targetClass);
	}

}
