package org.processmining.dataawarecnetminer.mining.decision;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Set;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.datadiscovery.ProjectedEvent;
import org.processmining.datadiscovery.ProjectedLog;
import org.processmining.datadiscovery.ProjectedTrace;
import org.processmining.log.utils.XUtils;

import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Iterators;
import com.google.common.collect.Sets;

final class ProjectedLogImpl implements ProjectedLog {

	private final class ProjectedEventForDecisionMining implements ProjectedEvent {
		private final int eventIndex;
		private final XEvent currentEvent;
		private final List<XEvent> augmentedTrace;
		private final XEvent previousEvent;

		private ProjectedEventForDecisionMining(int eventIndex, XEvent currentEvent, List<XEvent> augmentedTrace,
				XEvent previousEvent) {
			this.eventIndex = eventIndex;
			this.currentEvent = currentEvent;
			this.augmentedTrace = augmentedTrace;
			this.previousEvent = previousEvent;
		}

		public Set<String> getAttributes() {
			return Sets.filter(currentEvent.getAttributes().keySet(), new Predicate<String>() {

				public boolean apply(String key) {
					return consideredAttributes.contains(key);
				}
			});
		}

		public Object getAttributeValue(String attributeName) {
			XAttribute attribute = currentEvent.getAttributes().get(attributeName);
			if (attribute != null) {
				return XUtils.getAttributeValue(attribute);
			} else {
				return null;
			}
		}

		public Object getActivity() {
			return activityMapper.getActivity(currentEvent, previousEvent,
					augmentedTrace.subList(eventIndex, augmentedTrace.size()));
		}
	}

	private final class ProjectedTraceForDecisionMining implements ProjectedTrace {

		private final XTrace trace;

		private ProjectedTraceForDecisionMining(XTrace trace) {
			this.trace = trace;
		}

		public Iterator<ProjectedEvent> iterator() {
			final List<XEvent> augmentedTrace = new ArrayList<>(trace.size() + 2);
			augmentedTrace.add(artificalStartEvent);
			augmentedTrace.addAll(trace);
			augmentedTrace.add(artificalEndEvent);
			final ListIterator<XEvent> srcIter = augmentedTrace.listIterator();
			return new Iterator<ProjectedEvent>() {

				public ProjectedEvent next() {
					final int eventIndex = srcIter.nextIndex();
					final XEvent previousEvent = peekPrevious(srcIter);
					final XEvent currentEvent = srcIter.next();
					return new ProjectedEventForDecisionMining(eventIndex, currentEvent, augmentedTrace, previousEvent);
				}

				public boolean hasNext() {
					return srcIter.hasNext();
				}

				public void remove() {
					throw new UnsupportedOperationException();
				}

				private <T> T peekPrevious(ListIterator<T> iterator) {
					if (iterator.hasPrevious()) {
						T previous = iterator.previous();
						iterator.next();
						return previous;
					}
					return null;
				}

			};
		}

		public Set<String> getAttributes() {
			return Sets.intersection(consideredAttributes, trace.getAttributes().keySet());
		}

		public Object getAttributeValue(String attributeName) {
			return XUtils.getAttributeValue(trace.getAttributes().get(attributeName));
		}
	}

	private final ActivityMapper activityMapper;
	private final XEvent artificalStartEvent;
	private final XEvent artificalEndEvent;
	private final Set<String> consideredAttributes;
	private final Collection<XTrace> traces;

	ProjectedLogImpl(Collection<XTrace> traces, ActivityMapper activityMapper, XEvent artificalStartEvent,
			XEvent artificalEndEvent, Set<String> consideredAttributes) {
		this.traces = traces;
		this.activityMapper = activityMapper;
		this.artificalStartEvent = artificalStartEvent;
		this.artificalEndEvent = artificalEndEvent;
		this.consideredAttributes = consideredAttributes;
	}

	public Iterator<ProjectedTrace> iterator() {
		return Iterators.transform(traces.iterator(), new Function<XTrace, ProjectedTrace>() {

			public ProjectedTrace apply(final XTrace trace) {
				return new ProjectedTraceForDecisionMining(trace);
			}
		});
	}

	public Object getInitialValue(String attributeName) {
		return null;
	}

	public Set<String> getAttributes() {
		return ImmutableSet.of();
	}
}