package org.processmining.dataawarecnetminer.model;

import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutorService;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.model.XTrace;
import org.processmining.dataawarecnetminer.parallel.Command;
import org.processmining.dataawarecnetminer.parallel.TaskExecutor;
import org.processmining.datadiscovery.estimators.Type;
import org.processmining.datapetrinets.expression.GuardExpression;
import org.processmining.models.causalgraph.Relation;

import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Multiset;

public interface DataRelationStorage {

	public class Factory {

		public static DataRelationStorage createDataRelations(EventRelationStorage eventRelationStorage,
				Map<String, Class<?>> availableAttributes, final Set<String> consideredAttributes,
				ExecutorService executor) {

			final DataRelationStorage relationStorage = new DataRelationStorageImpl(eventRelationStorage,
					availableAttributes, consideredAttributes);

			// Build always to populate literal cache
			try {
				TaskExecutor.Factory.getInstance().execute(executor, eventRelationStorage.getAllTraces(),
						new Command<XTrace>() {

							public void execute(XTrace trace) {
								relationStorage.registerTrace(trace);
							}
						});
			} catch (InterruptedException e) {
				Thread.currentThread().interrupt();
			}

			return relationStorage;
		}

	}

	void registerTrace(XTrace trace);

	Multiset<Map<String, Object>> directlyFollowsUnderAttributes(Relation relation);

	DependencyRelation getL1RelationUnderCondition(Relation relation, GuardExpression condition);

	DependencyRelation getL2RelationUnderCondition(Relation relation, GuardExpression condition);

	DependencyRelation getDependencyRelationUnderCondition(Relation followsRelation, GuardExpression condition);

	int countOccurence(XEventClass source, GuardExpression condition);

	int countDirectlyFollows(Relation relation, GuardExpression condition);

	int countDirectlyFollows(XEventClass source, XEventClass target, GuardExpression condition);

	int countLengthTwoFollows(Relation relation, GuardExpression condition);

	int countLengthTwoFollows(XEventClass source, XEventClass target, GuardExpression condition);

	void setConsideredAttributes(Set<String> dataDiscoveryAttributes);

	ImmutableSet<String> getConsideredAttributes();

	Map<String, Set<String>> getLiteralValues();

	ImmutableMap<String, Type> getAttributeTypes();

	ImmutableMap<String, Class<?>> getAvailableAttributes();

}