package org.processmining.dataawarecnetminer.plugins;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.dataawarecnetminer.converter.CausalNetToPetrinet;
import org.processmining.datapetrinets.DataPetriNetsWithMarkings;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.dcnet.DataAwareCausalNet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.utils.ProvidedObjectHelper;

/**
 * Conversion from {@link DataAwareCausalNet} to
 * {@link DataPetriNetsWithMarkings}
 * 
 * @author F. Mannhardt
 *
 */
public final class DCNetToDPNPlugin {

	@Plugin(name = "Convert Data-aware Causal Net (DC-Net) to DPN", parameterLabels = { "Data-aware causal net" }, //
			level = PluginLevel.PeerReviewed, returnLabels = { "DPN" }, returnTypes = {
					DataPetriNetsWithMarkings.class }, userAccessible = true, mostSignificantResult = 1, help = "Converts a DataAwareCausalNet to a Data Petri net")
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public DataPetriNetsWithMarkings convertCNetToDPN(PluginContext context, DataAwareCausalNet dataAwareCausalNet) {
		DataPetriNetsWithMarkings dpn = convertCNetToDPN(dataAwareCausalNet);
		String label = ProvidedObjectHelper.getProvidedObjectLabel(context, dataAwareCausalNet);
		InitialMarkingConnection initialMarkingConnection = new InitialMarkingConnection(dpn, dpn.getInitialMarking());
		context.addConnection(initialMarkingConnection);
		ProvidedObjectHelper.publish(context, "Initial marking of " + label, dpn.getInitialMarking(), Marking.class,
				false);
		FinalMarkingConnection finalMarkingConnection = new FinalMarkingConnection(dpn, dpn.getFinalMarkings()[0]);
		context.addConnection(finalMarkingConnection);
		ProvidedObjectHelper.publish(context, "Final marking of " + label, dpn.getFinalMarkings()[0], Marking.class,
				false);
		context.getFutureResult(0).setLabel("DPN converted from " + label);
		return dpn;
	}

	public static DataPetriNetsWithMarkings convertCNetToDPN(DataAwareCausalNet dataAwareCausalNet) {
		return new CausalNetToPetrinet().convert(dataAwareCausalNet);
	}

}
