package org.processmining.dataawarecnetminer.util;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XCostExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XLog;
import org.processmining.datadiscovery.estimators.Type;
import org.processmining.log.utils.XUtils;
import org.xeslite.XLogMetadata;

import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.collect.Maps;
import com.google.common.collect.SetMultimap;

public class DataDiscoveryUtil {

	public static Map<String, Set<String>> viewAsMap(SetMultimap<String, String> multimap) {
		Map<String, Set<String>> literalsAsSets = Maps.transformValues(multimap.asMap(),
				new Function<Collection<String>, Set<String>>() {

					public Set<String> apply(Collection<String> set) {
						return (Set<String>) set; // those are in fact sets as they come from a SetMultimap
					}
				});
		return literalsAsSets;
	}

	public static Map<String, Type> viewSelectedOnly(Map<String, Type> attributeTypes,
			final Set<String> selectedAttributes) {
		// only the selected types
		return Maps.filterKeys(attributeTypes, new Predicate<String>() {

			public boolean apply(String key) {
				return selectedAttributes.contains(key);
			}
		});
	}

	public static Map<String, Class<?>> viewSelectedWithClass(Map<String, Class<?>> attributeTypes,
			final Set<String> selectedAttributes) {
		// only the selected classes
		return Maps.filterKeys(attributeTypes, new Predicate<String>() {

			public boolean apply(String key) {
				return selectedAttributes.contains(key);
			}
		});
	}

	public static Map<String, Class<?>> getRelevantAttributeTypes(XLog log) {
		Map<String, Class<?>> attributeTypes = getAttributesFromLog(log);
		for (Iterator<Entry<String, Class<?>>> iterator = attributeTypes.entrySet().iterator(); iterator.hasNext();) {
			Entry<String, Class<?>> entry = iterator.next();
			boolean standardAttribute = DataDiscoveryUtil.isStandardExtensionAttribute(entry.getKey());
			if (standardAttribute) {
				iterator.remove();
			}
		}
		return attributeTypes;
	}

	private static Map<String, Class<?>> getAttributesFromLog(XLog log) {
		Map<String, Class<?>> attributeTypes;
		if (log instanceof XLogMetadata) {
			attributeTypes = ((XLogMetadata) log).getAttributeTypes();
		} else {
			attributeTypes = new HashMap<>(XUtils.getEventAttributeTypes(log));
			attributeTypes.putAll(XUtils.getTraceAttributeTypes(log));
		}
		return attributeTypes;
	}

	public static boolean isStandardExtensionAttribute(String key) {
		// Lets hope that the JIT is clever enough to transform this to a hash table
		switch (key) {
			case XConceptExtension.KEY_NAME :
			case XConceptExtension.KEY_INSTANCE :
			case XTimeExtension.KEY_TIMESTAMP :
			case XLifecycleExtension.KEY_MODEL :
			case XLifecycleExtension.KEY_TRANSITION :
				//case XOrganizationalExtension.KEY_GROUP :
				//case XOrganizationalExtension.KEY_RESOURCE :
				//case XOrganizationalExtension.KEY_ROLE :
			case XCostExtension.KEY_AMOUNT :
			case XCostExtension.KEY_CURRENCY :
			case XCostExtension.KEY_DRIVER :
			case XCostExtension.KEY_TOTAL :
			case XCostExtension.KEY_TYPE :
				return true;
		}
		return false;
	}

}
