package org.processmining.dataawarecnetminer.visualization;

import java.awt.Dialog.ModalityType;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.text.NumberFormat;

import javax.swing.AbstractAction;
import javax.swing.JPopupMenu;
import javax.swing.JTextArea;

import org.apache.commons.lang3.StringUtils;
import org.processmining.dataawarecnetminer.model.DiscoveredDataAwareCausalNet;
import org.processmining.dataawarecnetminer.model.FrequencyAwareDataCausalNet;
import org.processmining.datadiscovery.estimators.FunctionEstimation;
import org.processmining.datapetrinets.expression.GuardExpression;
import org.processmining.models.cnet.CNetBinding;

public final class GuardExpressionNodeDecorator extends FrequencyNodeDecorator {

	private static final class ShowGuardAction extends AbstractAction {

		private static final long serialVersionUID = -5902923106298168763L;

		private final JPopupMenu menu;
		private final GuardExpression guard;
		private final CNetBinding binding;

		public ShowGuardAction(String name, JPopupMenu menu, CNetBinding binding, GuardExpression guard) {
			super(name);
			this.menu = menu;
			this.binding = binding;
			this.guard = guard;
		}

		public void actionPerformed(ActionEvent e) {
			JTextArea textArea = new JTextArea(guard.toPrettyString(1));
			textArea.setWrapStyleWord(true);
			textArea.setLineWrap(true);
			textArea.setFont(new Font("monospaced", Font.PLAIN, 12));
			textArea.setSize(1000, 800);
			UIUtil.showCustomNoHorizontalScoll(menu, textArea, "Guarded binding " + binding,
					ModalityType.MODELESS);
		}

	}

	private static final class ShowQualityAction extends AbstractAction {

		private static final long serialVersionUID = 8496698442299809924L;

		private final JPopupMenu menu;
		private final FunctionEstimation estimation;
		private final CNetBinding binding;

		public ShowQualityAction(String name, JPopupMenu menu, CNetBinding binding, FunctionEstimation estimation) {
			super(name);
			this.menu = menu;
			this.binding = binding;
			this.estimation = estimation;
		}

		public void actionPerformed(ActionEvent e) {
			JTextArea textArea = new JTextArea(estimation.toString());
			textArea.setWrapStyleWord(true);			
			textArea.setFont(new Font("monospaced", Font.PLAIN, 12));			
			UIUtil.showCustom(menu, textArea, "Guard quality " + binding,
					ModalityType.MODELESS);
		}

	}

	private final FrequencyAwareDataCausalNet frequencyAwareDCNet;
	private NumberFormat qualityMeasureFormatter = NumberFormat.getNumberInstance();

	public GuardExpressionNodeDecorator(FrequencyAwareDataCausalNet frequencyAwareDCNet) {
		super(frequencyAwareDCNet);
		this.frequencyAwareDCNet = frequencyAwareDCNet;
	}

	@Override
	public String getBindingLabel(CNetBinding binding) {
		String superLabel = super.getBindingLabel(binding);
		GuardExpression expression = getGuard(binding);
		if (expression != null) {
			StringBuilder sb = new StringBuilder(superLabel);
			sb.append(" |");
			sb.append(StringUtils.abbreviate(expression.toPrettyString(1), 200));
			FunctionEstimation estimation = getFunctionEstimation(binding);
			if (estimation != null) {
				sb.append(" ");
				sb.append('(');
				sb.append(qualityMeasureFormatter.format(estimation.getQualityMeasure()));
				sb.append(')');
			}
			return sb.toString();
		}
		return superLabel;
	}

	private GuardExpression getGuard(CNetBinding binding) {
		return frequencyAwareDCNet.getDCNet().getGuardExpression(binding);
	}

	private FunctionEstimation getFunctionEstimation(CNetBinding binding) {
		if (frequencyAwareDCNet.getDCNet() instanceof DiscoveredDataAwareCausalNet) {
			DiscoveredDataAwareCausalNet discoveredNet = (DiscoveredDataAwareCausalNet) frequencyAwareDCNet.getDCNet();
			return discoveredNet.getEstimation(binding);
		} else {
			return null;
		}
	}

	@Override
	public JPopupMenu getBindingMenu(CNetBinding binding) {
		JPopupMenu menu = super.getBindingMenu(binding);
		GuardExpression guard = getGuard(binding);
		if (guard != null) {
			menu.add(new ShowGuardAction("Show guard expression", menu, binding, guard));
		}
		FunctionEstimation estimation = getFunctionEstimation(binding);
		if (estimation != null) {
			menu.add(new ShowQualityAction("Show guard quality", menu, binding, estimation));
		}
		return menu;
	}
	
	

}